# # #
# /3/converting_bases.rb
# (c) 2009 Jan Lelis <mail@janlelis.de>. MIT License.
# See: https://rbjl.janlelis.com/3-converting-decimal-to-binary-integers-custom-methods-vs-to_i-update
# # #

# # # # # # # # #
# Test, which way to convert the base of an integer from 2 to 10 and vice versa,
# is the most efficient one

class ST
  attr_reader :dec_to_bin, :bin_to_dec, :correct

  # abstract methods
  def desc; end
  def speedtest_dec_to_bin; end
  def speedtest_bin_to_dec; end

  # stop time
  def timer(&proc)
    started_at = Time.now
    yield if block_given?
    stopped_at = Time.now

    res = stopped_at - started_at
    puts '%.2fs' % res
    res
  end

  # do the testing
  def speedtest( to_test = (42**42)**42*(42**42)**42 )
    @cur = to_test

    print 'Speed test: ' << desc << ' - decimal to binary...'
    @dec_to_bin = speedtest_dec_to_bin

    print 'Speed test: ' << desc << ' - binary to decimal...'
    @bin_to_dec = speedtest_bin_to_dec

    puts 'Methods ran correctly? ' << ((@correct = (@cur == to_test)) ? 'yes' : 'no')
  end
end

# ruby's .to_i and .to_s functions
class ST::To_i < ST
  def desc; 'using to_i and to_s'; end

  def speedtest_dec_to_bin
    timer{ @cur = @cur.to_s(2).to_i }
  end

  def speedtest_bin_to_dec
    timer{ @cur = @cur.to_s.to_i(2) }
  end
end

# calculate it
class ST::Calc < ST
  def desc; 'calculating it'; end

  def speedtest_dec_to_bin
    timer{
      res = []

      while @cur != 0
#       res << (@cur.odd? ? '1' : '0')
#       @cur >>= 1
        new_cur = @cur >> 1
        res << ((@cur == new_cur << 1) ? '0' : '1')
        @cur = new_cur
     end
     @cur = res.reverse.join.to_i
    }
  end

  def speedtest_bin_to_dec
    timer{
      res = 0
      cur_exp = 1
      cur_missing_1 = 0 # counts the 0s for shifting
      @cur.to_s.reverse.each_byte{ |set|
        if set == 49
          cur_exp <<= cur_missing_1
          res += cur_exp
          cur_missing_1 = 1
        else
          cur_missing_1 += 1
        end
      }
      @cur = res
    }
  end
end

# using sprintf
class ST::Sprintf < ST
  def desc; 'using sprintf'; end

  def speedtest_dec_to_bin
    timer{ @cur = ('%b' % @cur.to_s).to_i }
  end

  def speedtest_bin_to_dec
    timer{ @cur = ('%d' % ('0b' + @cur.to_s)).to_i }
  end
end

# for the lazy ones ;)
def ST.demo
  jl1 = ST::To_i.new
  jl1.speedtest

  jl2 = ST::Calc.new
  jl2.speedtest

  jl3 = ST::Sprintf.new
  jl3.speedtest
end

# let's start
ST.demo
